@@ -8,6 +8,7 @@
 #include <limits>
 #include <map>
 #include <optional>
+#include <unordered_map>
 
 /// Parse and store arguments
 class Arguments
@@ -55,7 +56,7 @@ public:
     /// Tell not to use the check byte derived from ciphertext entry metadata as known plaintext
     bool ignoreCheckByte = false;
 
-    /// Staring point of the attack on Z values remaining after reduction
+    /// Starting point of the attack on Z values remaining after reduction
     int attackStart = 0;
 
     /// Password from which to derive the internal password representation
@@ -112,6 +113,9 @@ public:
     /// \copydoc LengthInterval
     std::optional<LengthInterval> length;
 
+    /// Mask for password recovery, alternative to bruteforce and length
+    std::optional<std::vector<std::vector<std::uint8_t>>> mask;
+
     /// Starting point for password recovery
     std::string recoveryStart;
 
@@ -134,6 +138,16 @@ private:
     const char**       m_current;
     const char** const m_end;
 
+    std::unordered_map<char, std::bitset<256>> m_charsets;
+    std::unordered_map<char, std::string>      m_rawCharsets;
+
+    // Resolve the set of characters denoted by the given charset specification,
+    // recursively resolving referenced charsets from m_rawCharsets and caching results in m_charsets.
+    auto resolveCharset(const std::string& rawCharset) -> std::bitset<256>;
+
+    std::optional<std::string> m_rawBruteforce;
+    std::optional<std::string> m_rawMask;
+
     auto finished() const -> bool;
 
     void parseArgument();
@@ -161,6 +175,8 @@ private:
         bruteforce,
         length,
         recoverPassword,
+        mask,
+        charset,
         recoveryStart,
         jobs,
         exhaustive,
@@ -175,7 +191,7 @@ private:
     auto readSize(const std::string& description) -> std::size_t;
     auto readHex(const std::string& description) -> std::vector<std::uint8_t>;
     auto readKey(const std::string& description) -> std::uint32_t;
-    auto readCharset() -> std::vector<std::uint8_t>;
+    auto readRawCharset(const std::string& description) -> std::string;
 };
 
 #endif // BKCRACK_ARGUMENTS_HPP
